@@ -1,3 +1,5 @@
+import textwrap
+
 from ..procedures.procedure import Procedure
 from ..units.messenger import Messenger
 
@@ -9,7 +11,11 @@ class RunFunction(Procedure):
         self.kwargs = kwargs
 
     def run(self, messenger: Messenger):
-        messenger.info(f"Run function `{self.function.__name__}` with args: `{self.args}` and kwargs: `{self.kwargs}`")
+        args = textwrap.shorten(repr(self.args), width=100, placeholder="...")
+        kwargs = textwrap.shorten(repr(self.kwargs), width=100, placeholder="...")
+        messenger.info(f"Run function `{self.function.__name__}` "
+                       f"with args: `{args}` "
+                       f"and kwargs: `{kwargs}`")
 
         if hasattr(self.function, "__globals__"):
             self.function.__globals__["print"] = messenger.print  # inject messenger.print as print
